package Crypt::Store::Tie;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use Carp;
$VERSION = '0.01';

############################################################
sub TIEHASH
{
    my($package, %parms)=@_;

    my $self=bless {}, $package;

    $parms{HASH}||='Digest::MD5';
    $self->{HASH}=$parms{HASH}->new();
    $self->{CRYPTO}=$parms{CRYPTO}||'IDEA';
    $self->{STORE}=$parms{STORE};
    $self->{STORE}=tied %{$parms{STORE}} if tied %{$parms{STORE}};
    return $self;
}

############################################################
sub _mk_crypto
{
    my($self, $secret)=@_;
    return $self->{CRYPTO}->new($secret);
}
sub _mk_key
{
    my($self, $secret)=@_;
    $self->{HASH}->reset;
    $self->{HASH}->add($secret);
    return $self->{HASH}->hexdigest;
}

############################################################
sub store
{
    my($self)=@_;
    return $self->{STORE};
}

############################################################
sub keysize
{
    my($self)=@_;
    return $self->{CRYPTO}->keysize;
}

############################################################
sub blocksize
{
    my($self)=@_;
    return $self->{CRYPTO}->blocksize;
}

############################################################
sub FETCH
{
    my($self, $secret)=@_;
    # create the store's key
    my $key=$self->_mk_key($secret);

    # by-pass if we don't have the key
    return unless $self->{STORE}->EXISTS($key);
    # create a Crypt:: object
    my $crypto=$self->_mk_crypto($secret);

    # get data
    my $value=$self->{STORE}->FETCH($key);
    # warn unpack("H*", $secret).": Read ", length($value), " bytes";

    # find out what's padding, what's the data
    my $pad=unpack "n", $value;
    $value=substr($value, 2);       # 2 == length of an n
    my $data='';
    my $len=length($value);

    my $bs=$crypto->blocksize;
    my $q;
    # do the decryption loop
    for(my $q=0; $q < $len; $q+=$bs) {
        # warn "Decrypting ", length(substr $value, $q, $bs), " bytes\n";
        $data.=$crypto->decrypt(substr($value, $q, $bs));
    }
    substr($data, -$pad, $pad)='';  # truncate
    return $data;
}

############################################################
sub STORE
{
    my($self, $secret, $data)=@_;
    # create the store's key
    my $key=$self->_mk_key($secret);

    # create a Crypt:: object
    my $crypto=$self->_mk_crypto($secret);

    #################################################
    # encrypt data
    my $len=length $data;
    my $bs=$crypto->blocksize();
    my $pad=$bs-($len % $bs);
    $pad=0 if $pad == $bs;

    # first we start the number of bytes added as padding
    my $value=pack "n", $pad;
    # then pad out the data
    $data.="\xFF" x $pad;
    my $q;
    for($q=0; $q<$len; $q+=$bs) {
        # warn "-", substr($data, $q, $bs), "-";
        # warn length substr($data, $q, $bs);
        $value.=$crypto->encrypt(substr($data, $q, $bs));
    }

    # warn unpack("H*", $secret).": Saving ", length($value), " bytes";
    # save it
    $self->{STORE}->STORE($key, $value);

    # clear out data in memory PARANOID
#    substr($secret, 0, length $secret, 'X' x length $secret);
#    substr($data, 0, length $data, 'X' x length $data);
    return;     # nothing, so data doesn't leak anywhere
}

############################################################
sub DELETE
{
    my($self, $secret)=@_;

    # create the store's key
    my $key=$self->_mk_key($secret);

    # by-pass if we don't have the key
    return unless $self->{STORE}->EXISTS($key);

    # delete
    return unless $self->{STORE}->DELETE($key);
}

############################################################
sub CLEAR
{
    my($self)=@_;
    $self->{STORE}->CLEAR();
}

############################################################
sub EXISTS
{
    my($self, $secret)=@_;
    # create the store's key
    my $key=$self->_mk_key($secret);
    # ask the store if we have the key
    return $self->{STORE}->EXISTS($key);
}

############################################################
sub FIRSTKEY
{
    carp "You can't iterate over a cryptographic database.";
}
*NEXTKEY=\&FIRSTKEY;

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Crypt::Store::Tie - Cryptographic storage.  Tie interface.

=head1 SYNOPSIS

    use Crypt::Store::Tie;
    use DB_File;
    use Digest::MD5;
    use Crypt::IDEA 1.01;

    my %store;
    tie %store, 'DB_File', $new_filename, O_CREAT|O_RDWR;
    
    my %crypt;
    tie %crypt, 'Crypt::Store::Tie', 
                STORE=>\%store, 
                HASH=>'Digest::MD5',
                CRYPTO=>'IDEA';
                
    $crypt{$secret}="Some data";

    print $crypt{$secret2};


=head1 DESCRIPTION

Crypt::Store::Tie implements the cryptographic database algorythm.  Data is
stored as a secret and data.  The secret is hashed, creating a look up key.
The secret is also used to encrypt the data.  To access the data, the secret
must be given.  It is then hashed, to get the value, which is decrypted with
the secret.  This means that given cryptographicaly secure hashing and
encryption routines, only with knowledge of secret would an attacker be able
to access the data.  Also, this means that if you loose the secret, you will
not be able to access the data.

In psuedo-notation :

    S = secret
    D = data to be stored
    H() = hash routine
    E() = encryption routine
    D() = decryption routine

An attacker who has access to the database will only have access to H(S) and
E(H, D), which won't allow him to read the data.  Note, however, that the
attacker might be able to delete the data, so non-presence of the data in
the database is not a secure bit of information.

This module doesn't do the same thing as Crypt::xDBM_File.  Crypt::xDBM_File
encrypts all the data (keys and values) with a single secret.




=head1 TIE

The following parameters may be passed to tie :

=head2 STORE

Either a tied hash, or an object.  The following methods are called on 
the object : EXISTS, STORE, FETCH, DELETE, CLEAR.

=head2 HASH

Package to be used for the hashing routine.  Defaults to Digest::MD5. 
Caller must load the module.  The following methods are called: new, reset,
add, hexdigest.  Because hexdigest is used, the hashed key will be bigger,
but will also be usable as a file name. 

A new hashing object is created when the tieing is done.  Each time a secret
is to be hashed, reset/add/hexdigest is called in that order.


=head2 CRYPTO

Package to be used for cryptography.  Defaults to Digest::IDEA.  Caller must
load the module.  The following methods are called: new, encrypt, decrypt, 
blocksize.

A new crypto object is created each time you store or retrieve data from the
store.  It is up to you to make sure that the secret you use is an
acceptable key for the cryptographic algorythim you chose.  IDEA, for
instance, requires 16 byte keys.

=head1 METHODS

These are methods that may be called on the tied object.

=head2 store

Returns the tied object used for storing data.

=head2 blocksize

Returns the block size in bytes used by the crypto routine.

=head2 keysize

Returns the key size in bytes used by the crypto routine.  0 means variable
length.

=head1 AUTHOR

Philip Gwyn <cpan at pied.nu>

=head1 SEE ALSO

Crypt::Blowfish, Crypt::IDEA, Digest::MD5, Digest::SHA1, perltie, perl(1).

=cut
